import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, TicksPerSecond } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (!inventory) return;
    const container = inventory.container;
    if (!container) return;

    let registeredAddons = [];

    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (!itemStack) continue;

      const itemId = itemStack.typeId;
      const addonName = itemId.split(":")[0].replace("sg_", ""); // Extract addon name from typeId

      if (itemId.endsWith(":guide")) {
        // Add the tag
        const tag = `sgmanager:${addonName}`;
        if (!player.hasTag(tag)) {
          player.addTag(tag);
          registeredAddons.push(addonName);
        }
      }
    }

    if (registeredAddons.length > 0) {
      const visualNames = registeredAddons.map(getVisualName);
      const formattedNames = visualNames.slice(0, -1).join(", ") + 
        (visualNames.length > 1 ? " and " : "") + visualNames.slice(-1);
      player.sendMessage(`§aThe §e${formattedNames} Add-On${visualNames.length > 1 ? 's §awere' : ' §awas'} registered and added to your Essential Manager Handbook`);
      player.playSound("random.orb")
    }
  }
}, 2);

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_manager:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_manager:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential") && !lore.includes("§r§8Temporary")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});







world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_manager_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_manager_version", VERSION);
    if (player.hasTag("sg_manager_received_guide")) {
      player.removeTag("sg_manager_received_guide");
      player.sendMessage({
        translate: "sg_manager.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_manager:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_manager_version", VERSION);
    player.sendMessage({
      translate: "sg_manager.announce_update",
      with: [VERSION],
    });
  }
});

const addonNames = [
  { programmerName: "chop", visualName: "Tree Chop", button: "textures/sg/chop/items/guide.png"},
  { programmerName: "vein", visualName: "Vein Miner", button: "textures/sg/vein/items/guide.png"},
  { programmerName: "admin", visualName: "Admin", button: "textures/sg/admin/items/guide.png"},
  { programmerName: "warp", visualName: "Location Warps", button: "textures/sg/home/items/guide.png"},
  { programmerName: "tasks", visualName: "Daily Tasks", button: "textures/sg/tasks/items/guide.png"},
  { programmerName: "mail", visualName: "Mailbox", button: "textures/sg/mail/items/guide.png"},
  { programmerName: "trade", visualName: "Item Trading", button: "textures/sg/trade/items/guide.png"},
  { programmerName: "tpa", visualName: "TPA", button: "textures/sg/tpa/items/guide.png"},
  { programmerName: "light", visualName: "Dynamic Light", button: "textures/sg/light/items/guide.png"},
  // Add more as needed
];

const getVisualName = (programmerName) => {
  const found = addonNames.find(
    (addon) => addon.programmerName === programmerName
  );

  if (found) {
    return found.visualName;
  }

  // Replace underscores with spaces and capitalize each word
  return programmerName
    .split("_") // Split the string by underscores
    .map((word) => word.charAt(0).toUpperCase() + word.slice(1)) // Capitalize each word
    .join(" "); // Join the words back with spaces
};

const getIconPath = (programmerName) => {
  const found = addonNames.find(addon => addon.programmerName === programmerName);
  return found ? found.button : `textures/sg/${programmerName}/items/guide.png`; // Return null if no button path is found
};

const createManagerBook = (player) => {
  const tags = player.getTags();
  const addons = tags
    .filter((tag) => tag.startsWith("sgmanager:"))
    .map((tag) => tag.split(":")[1]) // Extract addon name
    .filter((addon) => addon !== "manager") // Ignore the manager addon
    .sort((a, b) => getVisualName(a).localeCompare(getVisualName(b))); // Sort by visual name

  const form = new ActionFormData().title("Essential Manager Add-On");
  form.body(
    `§e§lBEDROCK ESSENTIAL:§r\n§6The Essential Manager Add-On§r\nEasily view and access all Bedrock Essential Add-Ons Handbooks.\n§7You must collect the handbooks first to have them display here.`
  );

  addons.forEach((addon) => {
    const visualName = getVisualName(addon);
    const iconPath = getIconPath(addon);

    if (iconPath) {
      form.button(visualName, iconPath);
    } else {
      form.button(visualName); // No icon provided
    }
  });

  form.show(player).then((res) => {
    if (res.canceled) return;
    const selectedAddon = addons[res.selection];
    const guideId = `sg_${selectedAddon}:guide`;

    // Create the temporary item with additional lore
    const inventory = player.getComponent("inventory").container;
    const tempItem = new ItemStack(guideId, 1);
    tempItem.setLore(["§r§8Temporary"]); // Add the "Temporary" lore
    inventory.setItem(player.selectedSlotIndex, tempItem); // Place the item in the player's selected slot
    player.addTag("sg_manager:tempbook");

    // After 5 seconds, replace the "Temporary" item with the sg_manager:guide item
    system.runTimeout(() => {
      if (player.hasTag("sg_manager:tempbook")) {
        for (let i = 0; i < inventory.size; i++) {
          const itemStack = inventory.getItem(i);
          if (itemStack && itemStack.getLore().includes("§r§8Temporary")) {
            const guideItem = new ItemStack("sg_manager:guide", 1);
            inventory.setItem(i, guideItem); // Replace with the permanent item
            break; // Replace only one item
          }
        }
        player.removeTag("sg_manager:tempbook");
      }
    }, 3 * TicksPerSecond); // 3 seconds delay
  });
};




world.afterEvents.itemUse.subscribe((event) => {
  const player = event.source;
  const itemId = event.itemStack.typeId;
  if (itemId == "sg_manager:guide") {
    createManagerBook(player);
  }
  else if (itemId.endsWith("guide") && player.hasTag("sg_manager:tempbook") && event.itemStack.getLore().includes("§r§8Temporary")) {
    const inventory = player.getComponent("inventory").container;
    const guideItem = new ItemStack("sg_manager:guide", 1);
    inventory.setItem(player.selectedSlotIndex, guideItem); // Replace with the permanent item
    player.removeTag("sg_manager:tempbook");
  }
});
